/* -----------------------------------------------------------------------------
 * Antenna - An Ant-to-end solution for wireless Java 
 *
 * Copyright (c) 2002-2004 Joerg Pleumann <joerg@pleumann.de>
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * -----------------------------------------------------------------------------
 */
package de.pleumann.antenna;

import java.io.*;

import org.apache.tools.ant.*;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Reference;

import de.pleumann.antenna.misc.*;

public class WtkRun extends Task {

	private class Redirector implements Runnable {

		private InputStream input;

		private Task task;

		public Redirector(InputStream input, Task task) {
			this.input = input;
			this.task = task;
		}

		public void run() {
			int c;
			StringBuffer s = new StringBuffer("");

			try {
				while (((c = input.read()) != -1)) {
					if (c == '\n') {
						log(s.toString());
						s = new StringBuffer("");
					}
					else if (c != '\r') {
						s.append((char) c);
					}
				}
				input.close();

				if (s.length() != 0) {
					log(s.toString());
				}
			}
			catch (IOException ignored) {
			}
		}
	}

	private Utility utility;

    private Conditional condition;
    
	private Path classpath;

	private File jadFile;

	private String device;

	private String heapsize = "1M";

	private boolean wait = true;

    private String debug;

    private String trace;
        
	public void setClasspath(Path classpath) {
		if (this.classpath == null) {
			this.classpath = classpath;
		}
		else {
			this.classpath.append(classpath);
		}
	}

	public Path getClasspath() {
		return classpath;
	}

	public Path createClasspath() {
		if (classpath == null) {
			classpath = new Path(getProject());
		}
		return classpath.createPath();
	}

	public void setClasspathref(Reference r) {
		createClasspath().setRefid(r);
	}

	public void setDevice(String device) {
		this.device = device;
	}

	public void setHeapsize(String heapsize) {
		this.heapsize = heapsize;
	}

	public void setJadFile(File jadFile) {
		this.jadFile = jadFile;
	}

	public void setWait(boolean wait) {
		this.wait = wait;
	}

    public void setDebug(String address) {
        this.debug = address;
    }
    
    public void setTrace(String trace) {
        this.trace = trace;
    }
    
	public void init() throws BuildException {
		super.init();
		utility = Utility.getInstance(getProject(), this);
        condition = new Conditional(getProject());
	}

    public void setIf(String s) {
        condition.setIf(s);
    }
    
    public void setUnless(String s) {
        condition.setUnless(s);
    }

    public boolean isActive() {
        return condition.isActive();
    }

    private void executeSiemensEmulator(String device) throws BuildException {
        String emulator;
        
        if (device != null) {
	        emulator = utility.getWtkRelative("emulators/" + device + "/bin/emulator.exe");

	        if (!new File(emulator).exists()) {
	            emulator = utility.getWtkRelative(device + "emulators/bin/mmiu35.exe");
	        }

	        if (!new File(emulator).exists()) {
	            throw new BuildException("Siemens " + device + " emulator not found.");
	        }
        }
        else {
	        emulator = utility.getWtkRelative("/bin/emulator.exe");

	        if (!new File(emulator).exists()) {
	            throw new BuildException("Siemens emulator not found.");
	        }
        }
        
        emulator = utility.getQuotedName(new File(emulator));

        JadFile jad = new JadFile();
        try {
            jad.load(jadFile.getAbsolutePath(), null);
        }
        catch (IOException ex) {
            throw new BuildException("Unable to load JAD file", ex);
        }
        
        File jarFile = new File(jadFile.getParentFile(), jad.getValue("MIDlet-Jar-URL"));
                
        String arguments =
            (debug != null ? " /dj " : " /sj ") + utility.getQuotedName(jadFile);

        if (classpath != null) {
            arguments = arguments + " -classpath \"" + classpath + "\"";
        }

        if (trace != null) {
            arguments = arguments + " /tracing";
        }
                
        log("Executable: " + emulator, Project.MSG_VERBOSE);
        log("Arguments : " + arguments, Project.MSG_VERBOSE);

        try {
            Process proc = Runtime.getRuntime().exec(emulator + " " + arguments);

            if (wait) {
                proc.waitFor();
                if (proc.exitValue() != 0) {
                    throw new BuildException("Emulation failed (result=" + proc.exitValue() + ")");
                }
            }
        }
        catch (IOException ex) {
            throw new BuildException("Emulation failed", ex);
        }
        catch (InterruptedException ex) {
            throw new BuildException("Emulation failed", ex);
        }
    }

    private void executeIdenEmulator(String device) throws BuildException {
        String redirect = "";

        String emulator = utility.getWtkRelative("bin/emulator.exe");
//        if (!new File(emulator).exists()) {
//            emulator = utility.getWtkRelative("bin/emulator");
//            
//            if (!wait) {
//                log("Note: Calling the emulator with wait=false under Linux/Unix");
//                log("      might result in problems. If this is the case, please");
//                log("      send an e-mail to <joerg@pleumann.de>.");
//        
//                redirect = " </dev/nul >/dev/nul";
//            }
//        }

        emulator = utility.getQuotedName(new File(emulator));
        
        String arguments =
            "-Xdevice:" + device + " -Xheapsize:" + heapsize + " -Xdescriptor:" + utility.getQuotedName(jadFile) + redirect;

        if (classpath != null) {
            arguments = arguments + " -classpath \"" + classpath + "\"";
        }

        if (debug != null) {
            arguments = arguments + " -Xdebug -Xrunjdwp:transport=dt_socket,address=" + debug + ",server=y";
        }

        if (trace != null) {
            arguments = arguments + " -Xverbose:" + trace;
        }
                
        log("Executable: " + emulator, Project.MSG_VERBOSE);
        log("Arguments : " + arguments, Project.MSG_VERBOSE);

        try {
            Process proc = Runtime.getRuntime().exec(emulator + " " + arguments);

            if (wait) {
                new Thread(new Redirector(proc.getInputStream(), this)).start();
                new Thread(new Redirector(proc.getErrorStream(), this)).start();
                
                proc.waitFor();
                if (proc.exitValue() != 0) {
                    throw new BuildException("Emulation failed (result=" + proc.exitValue() + ")");
                }
            }
        }
        catch (IOException ex) {
            throw new BuildException("Emulation failed", ex);
        }
        catch (InterruptedException ex) {
            throw new BuildException("Emulation failed", ex);
        }
    }

    private void executeDefaultEmulator(String device) throws BuildException {
        String redirect = "";
        
        String emulator = utility.getWtkRelative(wait ? "bin/emulator.exe" : "bin/emulatorw.exe");
        if (!new File(emulator).exists()) {
            emulator = utility.getWtkRelative("bin/emulator");
            
            if (!wait) {
//                log("Note: Calling the emulator with wait=false under Linux/Unix");
//                log("      might result in problems. If this is the case, please");
//                log("      send an e-mail to <joerg@pleumann.de>.");
        
                redirect = " </dev/nul >/dev/nul";
            }
        }

        emulator = utility.getQuotedName(new File(emulator));
        
        String arguments =
            "-Xdevice:" + device + " -Xheapsize:" + heapsize + " -Xdescriptor:" + utility.getQuotedName(jadFile) + redirect;

        if (classpath != null) {
            arguments = arguments + " -classpath \"" + classpath + "\"";
        }

        if (debug != null) {
            arguments = arguments + " -Xdebug -Xrunjdwp:transport=dt_socket,address=" + debug + ",server=y";
        }

        if (trace != null) {
            arguments = arguments + " -Xverbose:" + trace;
        }
                
        log("Executable: " + emulator, Project.MSG_VERBOSE);
        log("Arguments : " + arguments, Project.MSG_VERBOSE);

        try {
            Process proc = Runtime.getRuntime().exec(emulator + " " + arguments);

            if (wait) {
                new Thread(new Redirector(proc.getInputStream(), this)).start();
                new Thread(new Redirector(proc.getErrorStream(), this)).start();
                
                proc.waitFor();
                if (proc.exitValue() != 0) {
                    throw new BuildException("Emulation failed (result=" + proc.exitValue() + ")");
                }
            }
        }
        catch (IOException ex) {
            throw new BuildException("Emulation failed", ex);
        }
        catch (InterruptedException ex) {
            throw new BuildException("Emulation failed", ex);
        }
    }
    
	public void execute() throws BuildException {
        if (!isActive()) return;
        
        if ("Siemens".equals(device) || utility.getToolkitType() == Utility.TOOLKIT_SIEMENS) {
            if ("Siemens".equals(device)) {
                device = null;
            }
            
            log("Running " + jadFile + " on Siemens " + (device == null ? "phone" : device));
            executeSiemensEmulator(device);
        }
        else if (utility.getToolkitType() == Utility.TOOLKIT_IDEN) {
            if (device == null) {
                device = "i85s";
            }

            log("Running " + jadFile + " on iDEN " + device);
            executeDefaultEmulator(device);
        }
        else {
            if (device == null) {
                device = "DefaultColorPhone";
            }
            
			log("Running " + jadFile + " on " + device);
            executeDefaultEmulator(device);
        }
    }
}
